import { Exponent } from "../exponent";
import { DivideUnits, ExponentiateUnit, MultiplyUnits, Unit } from "./types";
export declare class Measure<U extends Unit> {
    readonly value: number;
    private readonly unit;
    private readonly symbol?;
    static dimension<Dimension extends string>(dim: Dimension, symbol?: string): Measure<{ [K in Dimension]: [string, 1]; }>;
    static dimensionless(value: number): Measure<{}>;
    static of<U extends Unit>(value: number, quantity: Measure<U>, symbol?: string): Measure<U>;
    static unsafeConstruct<U extends Unit>(value: number, unit: U, symbol?: string): Measure<U>;
    private constructor();
    withSymbol(symbol: string): Measure<U>;
    getSymbol(): string | undefined;
    getUnit(): U;
    normalized(): Measure<U>;
    plus(other: Measure<U>): Measure<U>;
    minus(other: Measure<U>): Measure<U>;
    negate(): Measure<U>;
    scale(value: number | Measure<{}>): Measure<U>;
    times<V extends Unit>(other: Measure<V>): Measure<MultiplyUnits<U, V>>;
    over<V extends Unit>(other: Measure<V>): Measure<DivideUnits<U, V>>;
    per<V extends Unit>(other: Measure<V>): Measure<DivideUnits<U, V>>;
    toThe<N extends Exponent>(power: N): Measure<ExponentiateUnit<U, N>>;
    squared(): Measure<ExponentiateUnit<U, 2>>;
    cubed(): Measure<ExponentiateUnit<U, 3>>;
    inverse(): Measure<ExponentiateUnit<U, -1>>;
    reciprocal(): Measure<ExponentiateUnit<U, -1>>;
    compareTo(other: Measure<U>): number;
    isLessThan(other: Measure<U>): boolean;
    isLessThanOrEqualTo(other: Measure<U>): boolean;
    isEqualTo(other: Measure<U>): boolean;
    isNotEqualTo(other: Measure<U>): boolean;
    isGreaterThanOrEqualTo(other: Measure<U>): boolean;
    isGreaterThan(other: Measure<U>): boolean;
    toString(): string;
    in(unit: Measure<U>): string;
}
export declare function square<U extends Unit>(measure: Measure<U>): Measure<ExponentiateUnit<U, 2>>;
export declare function cubic<U extends Unit>(measure: Measure<U>): Measure<ExponentiateUnit<U, 3>>;
